package Tietorakenteet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

/**
 * @author MHo
 * @version 20.2.2020
 * Hiihtorekisterin jasenet, joka lisää ja poistaa jäseniä.
 */
public class Hiihdot {
    private ArrayList<Hiihto> hiihdot = new ArrayList<Hiihto>();
    
    /**
     * Muodostaja
     */
    public Hiihdot(){
       //
    }
    
    
    /**
     * @param hiihto suoritus, joka lisätään rekisteriin
     */
    public void lisaa(Hiihto hiihto){
        hiihdot.add(hiihto);
    }
    
    
    /**
     * Palauttaa viitteen i:teen jäseneen.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.anna(0) === eka;
     * hiihdot.anna(1) === toka;
     * </pre>  
     */
    public Hiihto anna(int i) throws IndexOutOfBoundsException {
        
        if (i < 0 || hiihdot.size() <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return hiihdot.get(i);
    }
    
    
    /**
    * Lukee harrastukset tiedostosta.
    * @throws SailoException jos lukeminen epäonnistuu
    * @example
    * <pre name="test">
    * #THROWS SailoException 
    * #import java.io.File;
    *  Hiihdot hiihdot = new Hiihdot();
    *  Hiihto h1 = new Hiihto(1);
    *  Hiihto h2 = new Hiihto(1);
    *  Hiihto h3 = new Hiihto(1); 
    *  Hiihto h4 = new Hiihto(2); 
    *  Hiihto h5 = new Hiihto(2); 
    *  File ftied = new File("hiihdot.dat");
    *  ftied.delete();
    *  hiihdot.lueTiedostosta(); #THROWS SailoException
    *  hiihdot.lisaa(h1);
    *  hiihdot.lisaa(h2);
    *  hiihdot.lisaa(h3);
    *  hiihdot.lisaa(h4);
    *  hiihdot.lisaa(h5);
    *  hiihdot.tallenna();
    *  hiihdot = new Hiihdot();
    *  hiihdot.lueTiedostosta();
    *  hiihdot.getHiihdot() === 5;
    *  hiihdot.lisaa(h1);
    *  hiihdot.tallenna();
    *  hiihdot.getHiihdot() === 6;
    *  ftied.delete() === true;
    *  File fbak = new File("hiihdot.bak");
    *  fbak.delete() === true;
    * </pre>
    */
    public void lueTiedostosta() throws SailoException {
        //setTiedostonPerusNimi(tied);
        try ( BufferedReader fi = new BufferedReader(new FileReader("hiihdot.dat")) ) {

            String rivi;
            while ( (rivi = fi.readLine()) != null ) {
                rivi = rivi.trim();
                if ( "".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                Hiihto h = new Hiihto(0);
                h.parse(rivi); // voisi olla virhekäsittely
                lisaa(h);
            }
            //muutettu = false;

        } catch ( FileNotFoundException e ) {
            throw new SailoException("Tiedosto 'hiihdot.dat' ei aukea");
        } catch ( IOException e ) {
            throw new SailoException("Ongelmia tiedoston kanssa: " + e.getMessage());
        }
    }


    /**
     * Tallentaa harrastukset tiedostoon.
     * @throws SailoException jos talletus epäonnistuu
     */
    public void tallenna() throws SailoException {
        //if ( !muutettu ) return;

        File fbak = new File("hiihdot.bak");
        File ftied = new File("hiihdot.dat");
        fbak.delete(); //  if ... System.err.println("Ei voi tuhota");
        ftied.renameTo(fbak); //  if ... System.err.println("Ei voi nimetä");

        try ( PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath())) ) {
            for (Hiihto h : hiihdot) {
                fo.println(h.toString());
            }
        } catch ( FileNotFoundException ex ) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch ( IOException ex ) {
            throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
        }

        //muutettu = false;
    }



    /**
     * @param id jasenen ID
     * @return tietyn jäsenen hiihdot listana
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(2);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.lisaa(eka);
     * hiihdot.lisaa(toka);
     * hiihdot.lisaa(kolmas);
     * hiihdot.getJasenenHiihdot(1) === "[eka, toka]";
     * hiihdot.getJasenenHiihdot(2) === "[kolmas]";
     * </pre>
     */
    public ArrayList<Hiihto> getJasenenHiihdot(int id) {
        ArrayList<Hiihto> jasenenHiihdot = new ArrayList<Hiihto>();
        for (int i = 0; i < hiihdot.size(); i++) {
            if (hiihdot.get(i).getJasenID() == id) jasenenHiihdot.add(hiihdot.get(i));
        }
        return jasenenHiihdot;
    }
    
    
    /**
     * @return rekisterin hiihtosuoritusten maara
     * @example
     * <pre name="test">
     * Hiihto eka = new Hiihto(1);
     * Hiihto toka = new Hiihto(1);
     * Hiihto kolmas = new Hiihto(3);
     * Hiihdot hiihdot = new Hiihdot();
     * hiihdot.lisaa(eka);
     * hiihdot.lisaa(toka);
     * hiihdot.lisaa(kolmas);
     * hiihdot.getHiihdot() === 3;
     * </pre>
     */
    public int getHiihdot() {
        return hiihdot.size();
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Hiihdot hiihdot = new Hiihdot();

        Hiihto lenkki = new Hiihto(1), toinen = new Hiihto(2);

        try {
            hiihdot.lisaa(lenkki);
            hiihdot.lisaa(toinen);

            System.out.println("========== Hiihdot testi ==============");

            for (int i=0; i < hiihdot.getHiihdot(); i++) {
                Hiihto hiihto = hiihdot.anna(i);
                System.out.println("Hiihto nro: " + i);
                hiihto.tulosta(System.out);
            }

        } catch ( Exception ex ) {
            System.out.println(ex.getMessage());
        }
    }
}